@@ -240,7 +240,15 @@ sentry_uuid_t
 sentry__envelope_get_event_id(const sentry_envelope_t *envelope)
 {
     if (envelope->is_raw) {
-        return sentry_uuid_nil();
+        const char *payload = envelope->contents.raw.payload;
+        size_t payload_len = envelope->contents.raw.payload_len;
+        const char *newline = memchr(payload, '\n', payload_len);
+        size_t header_len = newline ? (size_t)(newline - payload) : payload_len;
+        sentry_value_t header = sentry__value_from_json(payload, header_len);
+        sentry_uuid_t event_id = sentry_uuid_from_string(sentry_value_as_string(
+            sentry_value_get_by_key(header, "event_id")));
+        sentry_value_decref(header);
+        return event_id;
     }
     return sentry_uuid_from_string(sentry_value_as_string(
         sentry_value_get_by_key(envelope->contents.items.headers, "event_id")));
